;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x6e9ff7b9d4172538470aaa9278255245 . "./code.cg2js/cg-acache.fasl")
(#x12133f0c26ef3183c7de8e94ec544ee2 . "./code.cg2js/cg-base.fasl")
(#x8fab1f1dd23bb7a3cf8251f36db4b06e . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xc8291ba170564ebaab713ae5a0ea2d99 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xd1eb6eb2eb02a386edde287f7f72fcef . "./code.cg2js/cg-bitmap-stream.fasl")
(#xae37d64edbb245ae67572d9fff115cbc . "./code.cg2js/cg-button.fasl")
(#x7253a1a3bbdd6577848cd677139b3344 . "./code.cg2js/cg-calendar.fasl")
(#x5ea20d59534e062a49af219e507fcad8 . "./code.cg2js/cg-caret.fasl")
(#xde6486c57af893aca6a109683cf342ef . "./code.cg2js/cg-chart-or-plot.fasl")
(#x26bd8d46d42d00ce68e881e76fd584ae . "./code.cg2js/cg-chart-widget.fasl")
(#x70a80cda6e1ab02a838dcd755b53fd4d . "./code.cg2js/cg-check-box.fasl")
(#xede385ad067e7d7beed54b5bcf186da0 . "./code.cg2js/cg-choice-list.fasl")
(#x161643cda720d696efcc9636bc06201c . "./code.cg2js/cg-choose-printer.fasl")
(#xf9ca22962c56ce45623ac56f775b6634 . "./code.cg2js/cg-class-grid.fasl")
(#xe6200a52920b8cbf7825bfd6317d5dce . "./code.cg2js/cg-class-slot-grid.fasl")
(#xb72d4ff9c729fc3ad160c1bd1de85cb9 . "./code.cg2js/cg-class-support.fasl")
(#x9caab3df851e1238555b42afd3102ec2 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xea4543b2ae781a1c7a5b0281cb297c44 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x45e082d091155b21cd55dd0e6a06f922 . "./code.cg2js/cg-clipboard.fasl")
(#x057321fa17f50898a438f11647f4992a . "./code.cg2js/cg-color-dialog.fasl")
(#x08bbf3a388cffab92f7c753a44779eb4 . "./code.cg2js/cg-combo-box.fasl")
(#x2a99c298f2195ecd06c252594691df22 . "./code.cg2js/cg-common-control.fasl")
(#xc82fce17589618077504c6e54a1e1a16 . "./code.cg2js/cg-comtab.fasl")
(#xffd478ea5fe46a310b4f812b5d15fcb5 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xa5851e23c2c8dad72e8f34b3ee35a90c . "./code.cg2js/cg-curve.fasl")
(#xbb53db75a45896abd469bbda8f23b02c . "./code.cg2js/cg-dde-utils.fasl")
(#x0cb288a92094e48c7cdd1887aefed8cf . "./code.cg2js/cg-dev.fasl")
(#xd0a725355e80e2421466539e6778264e . "./code.cg2js/cg-dialog-item.fasl")
(#xe30e8328c26b5e7c6611ac2e4d930e32 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x17a876770bac1cdca4ac3b033a382767 . "./code.cg2js/cg-directory-dialog.fasl")
(#x50ec7bd62918f482d188566b5622b87c . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xbba53c178b1edefa3fa0703f3b39bd84 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x9939d315a0c29369465f46859a44805d . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xdd576b82069b0b46912293bfad566f9e . "./code.cg2js/cg-drawable.fasl")
(#x63e0ee3f9bbb4a15fa72ad43a9dfc201 . "./code.cg2js/cg-dropping-outline.fasl")
(#x9a985fbfecd2499cd1547a07fe44b0b2 . "./code.cg2js/cg-edit-in-place.fasl")
(#x632a468b189f916857f3cdacebd95f46 . "./code.cg2js/cg-editable-text.fasl")
(#x301fd8de870d1c41f64dd5e65c2b69a0 . "./code.cg2js/cg-file-dialog.fasl")
(#x4a80d15874fbe1f94e94d1e22318f4f5 . "./code.cg2js/cg-fill-texture.fasl")
(#x317770ab3b3305e7bbc9306ec3ee04b2 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x02c480cff67e0de9fad6dd493292246b . "./code.cg2js/cg-font-dialog.fasl")
(#xbb9c3abd690e118b67cd1c2bbe4c8d1c . "./code.cg2js/cg-gesture-emulation.fasl")
(#x315f803e8274e78c61f2b29137d1034e . "./code.cg2js/cg-get-pixmap.fasl")
(#x1558f55b6b646f4fe86f04f912c03d0b . "./code.cg2js/cg-get-position.fasl")
(#x59ff2f56da18ad0d057a183735b04f83 . "./code.cg2js/cg-graphics-context.fasl")
(#xd2ab3ec63fb7715923403fb4ac59946b . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xc52128b2deb380680bb5fdad46b55f89 . "./code.cg2js/cg-grid-widget.fasl")
(#x17d94f5f823be6216ff3c8aa054fb371 . "./code.cg2js/cg-group-box.fasl")
(#x61be4ff7deb83c6ed0aa739c921d0c7b . "./code.cg2js/cg-header-control.fasl")
(#xe25f9a45c077f68264c9c6e601943cc4 . "./code.cg2js/cg-hotspot.fasl")
(#x6d516202b8ec3a3e0d131abaf6559bdf . "./code.cg2js/cg-html-dialog.fasl")
(#x15c6e2138fb08aad346b85b747f8eb66 . "./code.cg2js/cg-html-widget.fasl")
(#xf12573d746619a3b381a92b8b7704d3d . "./code.cg2js/cg-icon-pixmap.fasl")
(#xcd8e03321c111fc79545090124c5f622 . "./code.cg2js/cg-icon.fasl")
(#xcc1b28a36fd82730120b419d23306cee . "./code.cg2js/cg-ie.fasl")
(#xebde592057cf7287d883845f9ce84f73 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xd11792220c2fc7b57dc668fa44e8eee0 . "./code.cg2js/cg-intersect.fasl")
(#x7b93b82025680ba135f0841edd23b953 . "./code.cg2js/cg-item-list.fasl")
(#x39c2156dc78e7a414af1bde830f37ff2 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xa1c581b4df6c4e2933e90a7ed7178534 . "./code.cg2js/cg-lamp.fasl")
(#xa589e23fdc2d9bfdb8f7b9128729effa . "./code.cg2js/cg-layout.fasl")
(#x5ec37a9317c9cbaa31a52886b0bf0d79 . "./code.cg2js/cg-lettered-menu.fasl")
(#x3e2ee0e700c32f136f535415f520bc7c . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xfffc1327ed274a031850b4c8fd82c710 . "./code.cg2js/cg-lisp-text.fasl")
(#x2bb76e70e2de1a1077bf2c99ee52d7c6 . "./code.cg2js/cg-lisp-widget.fasl")
(#xe6a11858878d22809d51cd4958e2cd09 . "./code.cg2js/cg-list-view.fasl")
(#x60a5f98cd20d815cff479dfa6d4c2a17 . "./code.cg2js/cg-mci.fasl")
(#x4d412e51cef5dcb9a0a4c5e817ef6909 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xd4bf482de105b8cc4dee6aae80234dc4 . "./code.cg2js/cg-menu.fasl")
(#x82a457fa641795a1e400a35b81026941 . "./code.cg2js/cg-message-dialog.fasl")
(#x029dd96fb987820ebeb0105f2b0439cf . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x08f6377c7efdb758bee13f995108e633 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xce82e219062a51a50fc88bd46740cbc5 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xe89d75056da00cce6bab9ab00f450ff5 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x126132ace1e585381567ca29935ab6bc . "./code.cg2js/cg-multi-picture-button.fasl")
(#xc8f04f8c670aff5e8f5553232946fc90 . "./code.cg2js/cg-nodes.fasl")
(#x2eb3225684a413eb3d9f226e6d6860b4 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xdb27fe42346ca272de5446ad11850339 . "./code.cg2js/cg-object-editor.fasl")
(#x014e6aac85987c1ac381d8742a7d7b07 . "./code.cg2js/cg-ocx.fasl")
(#xfdf07d5dac337eb966c7e61304d2c81c . "./code.cg2js/cg-os-widget.fasl")
(#x71f01469562834311663d6b717c49bce . "./code.cg2js/cg-os-window.fasl")
(#x345449f4b9a5a5097220c27bcfdbcc20 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x7eb75de937915c36470418616888e7b9 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x7e7867b5bbf0c8fba7e5f1e22f30759f . "./code.cg2js/cg-outline.fasl")
(#xb2fd040bca797c49539814a9eccb59fa . "./code.cg2js/cg-palette.fasl")
(#x2596d61dfb54ba0c59007e065f03696f . "./code.cg2js/cg-paren-matching.fasl")
(#xfba0ef8b499df1273e0f9046efc53f8b . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x6969849764109b2ca8f9f5eff40f7271 . "./code.cg2js/cg-picture-widget.fasl")
(#xc5c5ca9ae05b128c6fb0385a02c4d22c . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x51721ae68a677c180599f4d6af18e2c3 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x80c9868edba9bb18e8c24c718f7f8846 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x05b1840d52f6ddf0d7039481fdfd88b6 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x4a5904bca120294f074142672142a013 . "./code.cg2js/cg-pixmap.fasl")
(#x7cb08034a0cdff35a5b4bac6a6720247 . "./code.cg2js/cg-player.fasl")
(#x09ce77e46e4b734906ee4bb445ec3bfd . "./code.cg2js/cg-plot-widget.fasl")
(#x9917cf2e0b9d0b0bf373f50f68642e41 . "./code.cg2js/cg-printing.fasl")
(#x946330885183fdd894a8d47c7a1c96e8 . "./code.cg2js/cg-progress-indicator.fasl")
(#x9d9bddb1ba2bad99ed9e0a17ba57a691 . "./code.cg2js/cg-project-window.fasl")
(#x98c32ec47a7685020464600ba41be67a . "./code.cg2js/cg-property.fasl")
(#x7568420f2cc3b5c00cb46e94be0d9cd6 . "./code.cg2js/cg-radio-button.fasl")
(#xf49d7db024f12366b22de99749ebce19 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xd205487abdf21426f5d691e58ac41ba7 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x9aa524e3073ebb1bfd6e2442a20e1f55 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#xd59086049305c4949c56741cbab94708 . "./code.cg2js/cg-rich-edit.fasl")
(#xb71fafaf1a4c03ad4a6ed2709ddfc86d . "./code.cg2js/cg-sample-file-menu.fasl")
(#x5244928dde9430e1c4e789183641ef87 . "./code.cg2js/cg-scaling-stream.fasl")
(#x3a1cb1d3908765c2f68c58baa7dc3d9a . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xdd55cd3ff50d1cdbbdb22c08c8310688 . "./code.cg2js/cg-scroll-bar.fasl")
(#x66f87a36c4411ab8c5a06c453271f020 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x5abece42265197405dade5b5eb887c74 . "./code.cg2js/cg-selected-object.fasl")
(#x42cda1cca0e0fc027925a80d12c64e94 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xf435eb8ba9c0b6b839a970dd6409ac9c . "./code.cg2js/cg-split-bar.fasl")
(#xef486c6c877d7edb7f6fffd27f782946 . "./code.cg2js/cg-static-text.fasl")
(#x356be0f59d393fb382ac4f5507d5454c . "./code.cg2js/cg-status-bar.fasl")
(#x340a413504259f16a0e0340b1e0ecad5 . "./code.cg2js/cg-string-dialog.fasl")
(#xb744fd57162b94d87d949df43b7110e7 . "./code.cg2js/cg-tab-control.fasl")
(#x625590c0017825dc46036286270ccc8c . "./code.cg2js/cg-template-string.fasl")
(#xe30a299d5e0053bf66fdf07301ea1131 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xac0d44891b70dd0601d44973ce5ff517 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xfb954b8b322bbfb0183d8cc437b2e423 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xb4363570944b34c08c03a64405c26301 . "./code.cg2js/cg-text-or-combo.fasl")
(#xc9b6830cb36641097da814eb5ce7bd55 . "./code.cg2js/cg-text-widget.fasl")
(#x441fc5272572c72f5230eb925dbb890d . "./code.cg2js/cg-timer.fasl")
(#x96a56d4ea6ed721eb1ec3b27e8372512 . "./code.cg2js/cg-toggling-widget.fasl")
(#x93840fc4eb81297e6f816eba5cef33c4 . "./code.cg2js/cg-toolbar.fasl")
(#x585868a46d21f83edaaf254b339f6dca . "./code.cg2js/cg-tooltip.fasl")
(#xf5d359defa15e76ee1887a325efe2093 . "./code.cg2js/cg-touch.fasl")
(#x1e14b922744268cd6f6862d18543b5d6 . "./code.cg2js/cg-trackbar.fasl")
(#xbbd7060296c57191ed340ea2cd66a525 . "./code.cg2js/cg-tray.fasl")
(#x567c47e3c55e8848633e111cd1db0e3e . "./code.cg2js/cg-up-down-control.fasl")
(#xe4bac1341d3462b817d00d3d6e27e749 . "./code.cg2js/cg-utility-dialog.fasl")
(#xb9d8ad4b41138b565ea4934680ed183f . "./code.cg2js/cg-web-browser-dde.fasl")
(#x2746766a1d386a3011faf7f93cd73138 . "./code.cg2js/cg-web-browser.fasl")
(#xbc4c4245a89123ba46eab1d07fafe13e . "./code.cg2js/cg-wrap-string.fasl")
(#xf43ceb95f209a38de9e516df49d311c3 . "./code.cg2js/cg-yes-no-list.fasl")
(#x86011b33d760b28f625d26af6641c8d4 . "./code.cg2js/cg.fasl")
(#x7363a1d99dd988d03c7ce51748bb2b61 . "./code.cg2js/cg-yes-no-string.fasl")
(#xbe29656f5ca9eee74a32f81ecd5dc07c . "./code.idejs/ide-acache.fasl")
(#xc05c3cf27a99a89b8e9622875bab870b . "./code.idejs/ide-apropos.fasl")
(#x6cf972159ef493c27d841a97bfce5645 . "./code.idejs/ide-base.fasl")
(#x8a350a1407f0c569d14d10978383ac53 . "./code.idejs/ide-class-browser.fasl")
(#x12b6bb8b4c9375d674eb1cece2b0e260 . "./code.idejs/ide-class-grapher.fasl")
(#x0f0122ab17e81bd3ed9c0c33ad5ff343 . "./code.idejs/ide-clipboard.fasl")
(#x37ef4810abb9a7ab586079c16cfb7062 . "./code.idejs/ide-dev.fasl")
(#x996c79c8753b7b5f35a518cdc93419b8 . "./code.idejs/ide-editor.fasl")
(#x49e7697f1864b95b31481be8ce4c05f9 . "./code.idejs/ide-find-definitions.fasl")
(#x105a23ef2956bc2e250b1e5c3fc72c4d . "./code.idejs/ide-find-in-files.fasl")
(#x2833db99c026c8023703486d014c6a12 . "./code.idejs/ide-form.fasl")
(#xd0af317b56544786853dbd71140c2f69 . "./code.idejs/ide-grapher.fasl")
(#x99cb0b966d037682b091b48cc908eb7d . "./code.idejs/ide-inspector.fasl")
(#xb8ea49b8af1a670465c474c76f28e698 . "./code.idejs/ide-intro.fasl")
(#xc58abcd713b5b6940820eeb73fbfe3c0 . "./code.idejs/ide-keyshort.fasl")
(#xff9a7683d065d20a630986db234ab5c6 . "./code.idejs/ide-navigator.fasl")
(#xbe380417434c62b3562c6dd93da6ed14 . "./code.idejs/ide-package-list.fasl")
(#x506a2c1835eb55ccaa3620e5c37b0b9a . "./code.idejs/ide-patch-dialog.fasl")
(#x6c13b73fc36563e69c4a9ef5ef268ed8 . "./code.idejs/ide-process-browser.fasl")
(#xa096283cf9cff82a8643e07ae0d02948 . "./code.idejs/ide-project.fasl")
(#x77f09ef081d94020c1db7c6345507480 . "./code.idejs/ide-trace.fasl")
(#x3ba54da658f93c54da086b4bbc65cdf3 . "./code.idejs/ide-window-list.fasl")
(#xc2ed7cf9684d2095b746e772e78e1d3a . "./code.idejs/ide.fasl")
(#x68149b79128815fb04933c4813dbe32f . "./cgjs.html")
(#x55fea08d92e113f5a8381c271febbc9e . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
